// disable warning message about converting double constant to float constant
#pragma warning(disable:4305)

#include <modelerdll.h>
#include <math.h>

#define BASE_ROTATION   1
#define LOWER_TILT      2
#define UPPER_TILT      3
#define UPPER_TWIST     4

MODEL("RobotArm");

static GLuint texName;
unsigned char *textureImage;
int imageWidth;
int imageHeight;

bool init(void* args)
{
    scale( BASE_ROTATION, "base rotation", 0.0, 360.0, 0.1, 30.0 );
    scale( LOWER_TILT, "lower arm tilt", -60.0, 60.0, 0.1, 0.0 );
    scale( UPPER_TILT, "upper arm tilt", -120.0, 120.0, 0.1, 0.0 );
    scale( UPPER_TWIST, "upper arm twist", -45.0, 45.0, 0.1, 0.0 );
	
    return true;
}


void redraw(int drawmode)
{
 
	static bool initialized = false; // set to true the first time redraw is called 
	
	if (!initialized) { 
		initialized = true; 
		printf("Vendor version %s %s\n",glGetString(GL_VENDOR),glGetString(GL_VERSION)); 
		
		// texture preparation
		
		textureImage = readBMP("BoxSurface.bmp", imageWidth, imageHeight);  // must be powers of 2
		if (textureImage == NULL) {
			printf("texture image is null\n");
		} else {
//			printf("texture image is %3i x %3i\n",imageWidth,imageHeight);
			
			glPixelStorei(GL_UNPACK_ALIGNMENT,4);	// readBMP returns rows aligned to 4 bytes
			
			glGenTextures(1,&texName);
			glBindTexture(GL_TEXTURE_2D,texName);
			
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST);
			
			glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,imageWidth,imageHeight,0,GL_RGB,
				GL_UNSIGNED_BYTE,textureImage);


			GLenum error_flag; 
			error_flag = glGetError(); 
			if (error_flag != GL_NO_ERROR) { 
				printf("Error:  %1s (%i) in %1s.\n",gluErrorString(error_flag),error_flag,"redraw initialization"); 
			} 
			
		}
		
	} 

	//	printf("time is %f\n", get_time());

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
    /* define the viewing transform:  orient the model so that +z is
	up and +x is to the right. */
	
    glMatrixMode( GL_MODELVIEW );
    glTranslated( 0.0, -4.0, 0);
	
    glRotated( -90.0, 1.0, 0.0, 0.0 );

	/* establish a textured floor */

    glPushMatrix();
    glTranslated( -5, -5, -1 );
	diffuse_color( 0.3, 0.3, 0.3 );
    box( 10.0, 10.0, 1.0 );
    glPopMatrix();
				
    /* turn the whole assembly around the z-axis. */
	
    glRotated( get_control_d( BASE_ROTATION ), 0.0, 0.0, 1.0 );
	
    /* place a box with the center of its bottom face at the origin. */
	
    glPushMatrix();
    glTranslated( -1.5, -1.0, 0.0 );
	diffuse_color( 0.6, 0.6, 0.6 );
//    box( 3.0, 2.0, 1.0 );
    textureBox( 3.0, 2.0, 1.0 );
    glPopMatrix();
	
    /* move the origin to the top of the box, slightly offset from the
	center. */
	
    glTranslated( 0.5, 0.0, 1.0 );
	
    /* turn the coordinate system on the y-axis to tilt the lower arm
	around its base. */
	
    glPushMatrix();
    glRotated( get_control_d( LOWER_TILT ), 0.0, 1.0, 0.0 );
	
    /* place the cylinder that the lower arm is mounted on. */
	
    glPushMatrix();
	/* want a cylinder oriented along y, so turn coordinates 90
	degrees, making z into y. */
	
	glRotated( 90.0, 1.0, 0.0, 0.0 );
	
	/* translate so that the cylinder is centered at the origin. */
	
	glTranslated( 0.0, 0.0, -0.5 );
    
	diffuse_color( 0.6, 0.3, 0.3 );
	cylinder( 1.0, 0.5, 0.5 );
    glPopMatrix();
	
    /* place the lower arm, a box with its bottom face centered at the
	origin. */
	
    glPushMatrix();
	glTranslated( -0.25, -0.25, 0.0 );
	diffuse_color( 0.6, 0.6, 0.6 );
	box( 0.5, 0.5, 3.0 );
    glPopMatrix();
	
    /* move the coordinate system to the center of the lower arm's
	upper face. */
	
    glPushMatrix();
	glTranslated( 0.0, 0.0, 3.0 );
	
	/* twist the upper arm around the axis of the lower arm. */
	
	glRotated( get_control_d( UPPER_TWIST ), 0.0, 0.0, 1.0 );
	
	/* place a sphere at the joint. */
	
	diffuse_color( 0.6, 0.3, 0.3 );
	sphere( 0.5 );

	/* place another upper arm, twisted an equal amount in the
	opposite direction. */
	
	glPushMatrix();
	glRotated( -get_control_d( UPPER_TILT ), 0.0, 1.0, 0.0 );
	diffuse_color( 0.7, 0.7, 0.7 );
	cylinder( 3.0, 0.25, 0.0 );
	glPopMatrix();
    
	/* place an upper arm, twisted down around the y axis. */ 
	
	glPushMatrix();
	glRotated( get_control_d( UPPER_TILT ), 0.0, 1.0, 0.0 );
	cylinder( 3.0, 0.25, 0.0 );

	/*	add a triangle based do-dad thing on top to show how
	a simple animation can be done.  */
	
	glPushMatrix();

	glTranslated( 0.0, 0.0, 3.0 );
	ambient_color( 0.0, 0.0, 0.4 );
	diffuse_color( 0.0, 0.4, 0.7 );
	glRotated( get_time() * 90.0, 0.0, 0.0, 1.0 );

	triangle(0.0,0.0,0.0, 2.0,0.0,0.5, 0.0,0.0,1.0);
	triangle(0.0,0.0,0.0, 0.0,0.0,1.0, 0.0,2.0,0.5);
	triangle(0.0,0.0,0.0, 0.0,2.0,0.5, 2.0,0.0,0.5);
	triangle(0.0,0.0,1.0, 2.0,0.0,0.5, 0.0,2.0,0.5);

	glPopMatrix();  // tip of the upper arm
	
	glPopMatrix();	// tip of the lower arm
	
    glPopMatrix();	// bottom of the lower arm
    
    glPopMatrix();	// top of the base
	
	
}

